
#include <assert.h>
#include "csearch.h"

#define MAX_NODES_IN_EXHAUSTIVE_SEARCH	30

static int exhaustive_search_assign(Problem * p, 
			unsigned i, unsigned * region_size)
{
	unsigned N_1 = p->num_nodes - 1;
	unsigned num_regions = 1;
	unsigned j, r;

	region_size[0] = p->nodes[0].size;
	p->nodes[0].region = 0;
	for (j = 0; j < N_1; j++) {
		if ((i >> j) & 1) {
			r = num_regions;
			region_size[r] = 0;
			num_regions++;
		} else {
			r = p->nodes[p->nodes[j+1].parent].region;
		}
		p->nodes[j+1].region = r;
		region_size[r] += p->nodes[j+1].size;
		if (region_size[r] > p->spm_size) {
			return 0;
		}
	}
	return 1;
}

uint64_t exhaustive_search(Problem * p)
{
	unsigned N_1 = p->num_nodes - 1;
	unsigned i, j, end, best_i = 0;
	unsigned region_size[MAX_NODES_IN_EXHAUSTIVE_SEARCH + 1];
	uint64_t value, best_value = ~0;

	assert(p->num_nodes < MAX_NODES_IN_EXHAUSTIVE_SEARCH);
	end = 1 << N_1;

	for (i = 0; i < end; i++) {
		if (!exhaustive_search_assign(p, i, region_size)) {
			continue;
		}

#if 0
        if (p->mode == 'P') {
            value = p->fixed_cost;
        } else {
            value = (region_size[0] + p->fixed_cost);
        }

		for (j = 1; j <= N_1; j++) {
            Node * nc = &p->nodes[j];
            Node * np = &p->nodes[nc->parent];
			unsigned ncr = nc->region;
			unsigned npr = np->region;

			if (npr != ncr) {
				if (p->mode == 'P') {
					value += (uint64_t) nc->edge_weight
                            * (uint64_t) p->fixed_cost;
				} else {
					value += (uint64_t) nc->edge_weight *
                        (uint64_t) (p->fixed_cost + region_size[ncr]);
                    if (!nc->edge_is_sequential) {
                        value += (uint64_t) nc->edge_weight *
                            (uint64_t) (p->fixed_cost + region_size[npr]);
                    }
				}
			}
		}
#endif
        value = validate(p);
		if (value < best_value) {
			best_value = value;
			best_i = i;
		}
	}
	exhaustive_search_assign(p, best_i, region_size);
	return best_value;
}


