#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdint.h>

#define CALL_FLAG 1


int main(int argc, char ** argv)
{
    FILE *          input;
    FILE *          output;
    const char *    system_cpu = "system.cpu ";
    int             system_cpu_len = strlen(system_cpu);
    int             enable = 0;
    unsigned long long pc;
    unsigned long long main_at;
    uint32_t        pc4;
    uint64_t        counter = 0;
    char            buf[BUFSIZ];

    if (argc != 4) {
        fprintf(stderr, "Expected three arguments.\n");
        exit(1);
    }
    input = fopen(argv[1], "rt");
    if (!input) {
        fprintf(stderr, "Unable to open input file.\n");
        exit(1);
    }
    output = fopen(argv[2], "wb");
    if (!output) {
        fprintf(stderr, "Unable to open output file.\n");
        exit(1);
    }
    main_at = strtoull(argv[3], NULL, 16);
    if ((main_at == 0) || (main_at & 3) || (main_at >= (1ULL << 32ULL))) {
        fprintf(stderr, "Invalid main location.\n");
        exit(1);
    }

    while (fgets(buf, BUFSIZ - 1, input)) {
        char * p2;

        if (strncmp(buf, system_cpu, system_cpu_len) != 0) {
            fprintf(stderr, "Line format error 1.\n%s", buf);
            exit(1);
        }
        pc = strtoull(&buf[system_cpu_len], &p2, 16);
        if ((pc == 0) || (pc & 3) || (pc >= (1ULL << 32ULL))) {
            fprintf(stderr, "Line format error 2.\n%s", buf);
            exit(1);
        }
        if (pc == main_at) {
            enable = 1;
        }
        if (!(counter & 0xffff)) {
            fprintf(stderr, "%ju %d\n", counter, enable);
        }
        if (enable) {
            pc4 = pc;
            const char * bl = strstr(p2, " bl");
            if (bl && (bl[3] == ' ' || bl[3] == 'x')) {
                pc4 |= CALL_FLAG;
            }
            fwrite(&pc4, 4, 1, output);
        }
        counter ++;
    }
    fclose(output);
    fclose(input);
    if (!enable) {
        fprintf(stderr, "Never reached main function.\n");
        exit(1);
    }
    return 0;
}

